/** \file dspin_defs.h
 *
 * Gathers various definitions used by DSpin class.
 *
 * They have been placed in a separate file so that no to clutter the class header file.
 *
 *  Created on: Jun 16, 2012
 *      Author: Eric Pascual (eric@pobot.org)
 */

#ifndef DSPIN_DEFS_H_
#define DSPIN_DEFS_H_

/**
 * dSpin registers.
 *
 * Enum members names are built by prepending "dSPIN_REG_" to the register name,
 * as it appears in the datasheet.
 */
typedef enum {
	dSPIN_REG_ABS_POS = 0x01,
	dSPIN_REG_EL_POS = 0x02,
	dSPIN_REG_MARK = 0x03,
	dSPIN_REG_SPEED = 0x04,
	dSPIN_REG_ACC = 0x05,
	dSPIN_REG_DEC = 0x06,
	dSPIN_REG_MAX_SPEED = 0x07,
	dSPIN_REG_MIN_SPEED = 0x08,
	dSPIN_REG_KVAL_HOLD = 0x09,
	dSPIN_REG_KVAL_RUN = 0x0A,
	dSPIN_REG_KVAL_ACC = 0x0B,
	dSPIN_REG_KVAL_DEC = 0x0C,
	dSPIN_REG_INT_SPD = 0x0D,
	dSPIN_REG_ST_SLP = 0x0E,
	dSPIN_REG_FN_SLP_ACC = 0x0F,
	dSPIN_REG_FN_SLP_DEC = 0x10,
	dSPIN_REG_K_THERM = 0x11,
	dSPIN_REG_ADC_OUT = 0x12,
	dSPIN_REG_OCD_TH = 0x13,
	dSPIN_REG_STALL_TH = 0x14,
	dSPIN_REG_FS_SPD = 0x15,
	dSPIN_REG_STEP_MODE = 0x16,
	dSPIN_REG_ALARM_EN = 0x17,
	dSPIN_REG_CONFIG = 0x18,
	dSPIN_REG_STATUS = 0x19
} dSPIN_REGISTER;

/**
 * dSpin commands.
 *
 * Enum members names are built by prepending "dSPIN_CMD_" to the register name,
 * as it appears in the datasheet.
 */
typedef enum {
	dSPIN_CMD_NOP = 0x00,
	dSPIN_CMD_SET_PARAM = 0x00,
	dSPIN_CMD_GET_PARAM = 0x20,
	dSPIN_CMD_RUN = 0x50,
	dSPIN_CMD_STEP_CLOCK = 0x58,
	dSPIN_CMD_MOVE = 0x40,
	dSPIN_CMD_GOTO = 0x60,
	dSPIN_CMD_GOTO_DIR = 0x68,
	dSPIN_CMD_GO_UNTIL = 0x82,
	dSPIN_CMD_RELEASE_SW = 0x92,
	dSPIN_CMD_GO_HOME = 0x70,
	dSPIN_CMD_GO_MARK = 0x78,
	dSPIN_CMD_RESET_POS = 0xD8,
	dSPIN_CMD_RESET_DEVICE = 0xC0,
	dSPIN_CMD_SOFT_STOP = 0xB0,
	dSPIN_CMD_HARD_STOP = 0xB8,
	dSPIN_CMD_SOFT_HIZ = 0xA0,
	dSPIN_CMD_HARD_HIZ = 0xA8,
	dSPIN_CMD_GET_STATUS = 0xD0
} dSPIN_COMMAND;

/**
 * Over-current detection thresholds.
 */
typedef enum {
	dSPIN_OCD_TH_375mA = 0x00,
	dSPIN_OCD_TH_750mA = 0x01,
	dSPIN_OCD_TH_1125mA = 0x02,
	dSPIN_OCD_TH_1500mA = 0x03,
	dSPIN_OCD_TH_1875mA = 0x04,
	dSPIN_OCD_TH_2250mA = 0x05,
	dSPIN_OCD_TH_2625mA = 0x06,
	dSPIN_OCD_TH_3000mA = 0x07,
	dSPIN_OCD_TH_3375mA = 0x08,
	dSPIN_OCD_TH_3750mA = 0x09,
	dSPIN_OCD_TH_4125mA = 0x0A,
	dSPIN_OCD_TH_4500mA = 0x0B,
	dSPIN_OCD_TH_4875mA = 0x0C,
	dSPIN_OCD_TH_5250mA = 0x0D,
	dSPIN_OCD_TH_5625mA = 0x0E,
	dSPIN_OCD_TH_6000mA = 0x0F
} dSPIN_OVER_CURRENT_THRESHOLD;

/** \name STEP_MODE register */
///@{

/// SYNC_EN field mask
#define dSPIN_STEP_MODE_SYNC_EN	0x80
/// SYNC_SEL field mask
#define dSPIN_STEP_MODE_SYNC_SEL 0x70
/// WRITE field mask
#define dSPIN_STEP_MODE_WRITE 	0x08
/// STEP_SEL field mask
#define dSPIN_STEP_MODE_STEP_SEL 0x07

/**
 * Step modes.
 */
typedef enum {
	dSPIN_STEP_SEL_1 = 0x00,
	dSPIN_STEP_SEL_1_2 = 0x01,
	dSPIN_STEP_SEL_1_4 = 0x02,
	dSPIN_STEP_SEL_1_8 = 0x03,
	dSPIN_STEP_SEL_1_16 = 0x04,
	dSPIN_STEP_SEL_1_32 = 0x05,
	dSPIN_STEP_SEL_1_64 = 0x06,
	dSPIN_STEP_SEL_1_128 = 0x07
} dSPIN_STEP_SEL;

/**
 * Sync modes.
 */
typedef enum {
	dSPIN_SYNC_SEL_1_2 = 0x00,
	dSPIN_SYNC_SEL_1 = 0x10,
	dSPIN_SYNC_SEL_2 = 0x20,
	dSPIN_SYNC_SEL_4 = 0x30,
	dSPIN_SYNC_SEL_8 = 0x40,
	dSPIN_SYNC_SEL_16 = 0x50,
	dSPIN_SYNC_SEL_32 = 0x60,
	dSPIN_SYNC_SEL_64 = 0x70
} dSPIN_SYNC_SEL;

/**
 * Sync enabling.
 */
typedef enum {
	dSPIN_SYNC_EN = 0x80,
	dSPIN_SYNC_DIS = 0x00
} dSPIN_SYNC_ENABLING;

///@}

/**
 * ALARM_EN register flags.
 */
typedef enum {
	dSPIN_ALARM_EN_OVERCURRENT = 0x0,
	dSPIN_ALARM_EN_THERMAL_SHUTDOWN = 0x0,
	dSPIN_ALARM_EN_THERMAL_WARNING = 0x0,
	dSPIN_ALARM_EN_UNDER_VOLTAGE = 0x0,
	dSPIN_ALARM_EN_STALL_DET_A = 0x1,
	dSPIN_ALARM_EN_STALL_DET_B = 0x2,
	dSPIN_ALARM_EN_SW_TURN_ON = 0x4,
	dSPIN_ALARM_EN_WRONG_NPERF_CMD = 0x80
} dSPIN_ALARM_EN;

/** \name CONFIG register */
///@{

/// OSC_SEL field mask
#define dSPIN_CONFIG_OSC_SEL                 0x000F
/// SW_MODE field mask
#define dSPIN_CONFIG_SW_MODE                 0x0010
/// EN_VSCOMP field mask
#define dSPIN_CONFIG_EN_VSCOMP               0x0020
/// OC_SD field mask
#define dSPIN_CONFIG_OC_SD                   0x0080
/// POW_SR field mask
#define dSPIN_CONFIG_POW_SR                  0x0300
/// F_PWM_DEC field mask
#define dSPIN_CONFIG_F_PWM_DEC               0x1C00
/// F_PWM_INT field mask
#define dSPIN_CONFIG_F_PWM_INT               0xE000

/**
 * Oscillator selectors
 */
typedef enum {
	/// Internal 16MHz, no output
	dSPIN_OSC_SEL_INT_16MHZ = 0x0000,
	/// Default; internal 16MHz, 2MHz output
	dSPIN_OSC_SEL_INT_16MHZ_OSCOUT_2MHZ = 0x0008,
	/// Internal 16MHz, 4MHz output
	dSPIN_OSC_SEL_INT_16MHZ_OSCOUT_4MHZ = 0x0009,
	/// Internal 16MHz, 8MHz output
	dSPIN_OSC_SEL_INT_16MHZ_OSCOUT_8MHZ = 0x000A,
	/// Internal 16MHz, 16MHz output
	dSPIN_OSC_SEL_INT_16MHZ_OSCOUT_16MHZ = 0x000B,
	/// External 8MHz crystal
	dSPIN_OSC_SEL_EXT_8MHZ_XTAL_DRIVE = 0x0004,
	/// External 16MHz crystal
	dSPIN_OSC_SEL_EXT_16MHZ_XTAL_DRIVE = 0x0005,
	/// External 24MHz crystal
	dSPIN_OSC_SEL_EXT_24MHZ_XTAL_DRIVE = 0x0006,
	/// External 32MHz crystal
	dSPIN_OSC_SEL_EXT_32MHZ_XTAL_DRIVE = 0x0007,
	/// External 8MHz crystal, output inverted
	dSPIN_OSC_SEL_EXT_8MHZ_OSCOUT_INVERT = 0x000C,
	/// External 16MHz crystal, output inverted
	dSPIN_OSC_SEL_EXT_16MHZ_OSCOUT_INVERT = 0x000D,
	/// External 24MHz crystal, output inverted
	dSPIN_OSC_SEL_EXT_24MHZ_OSCOUT_INVERT = 0x000E,
	/// External 32MHz crystal, output inverted
	dSPIN_OSC_SEL_EXT_32MHZ_OSCOUT_INVERT = 0x000F
} dSPIN_OSC_SEL;

/**
 * Switch signal usage
 */
typedef enum {
	/// Default; hard stop motor on switch.
	dSPIN_SW_MODE_HARD_STOP = 0x0000,
	/// Tie to the GoUntil and ReleaseSW
	dSPIN_SW_MODE_USER = 0x0010
} dSPIN_SW_MODE;

/**
 * Motor voltage compensation mode
 */
typedef enum {
	/// Disable motor voltage compensation.
	dSPIN_VS_COMP_DISABLE = 0x0000,
	/// Enable motor voltage compensation.
	dSPIN_VS_COMP_ENABLE = 0x0020
} dSPIN_VS_COMP;

/**
 * Over-current shutdown settings
 */
typedef enum {
	/// Bridges do NOT shutdown on OC detect
	dSPIN_OC_SD_DISABLE = 0x0000,
	/// Bridges shutdown on OC detect
	dSPIN_OC_SD_ENABLE = 0x0080
} dSPIN_OC_SD;

/**
 * Power slew-rate settings
 */
typedef enum {
	/// 180V/us
	dSPIN_POW_SR_180V_us = 0x0000,
	/// 290V/us
	dSPIN_POW_SR_290V_us = 0x0200,
	/// 530V/us
	dSPIN_POW_SR_530V_us = 0x0300
} dSPIN_POW_SR;

/**
 * PWM clock divider
 */
typedef enum {
	dSPIN_PWM_DIV_1 = (0x00) << 13,
	dSPIN_PWM_DIV_2 = (0x01) << 13,
	dSPIN_PWM_DIV_3 = (0x02) << 13,
	dSPIN_PWM_DIV_4 = (0x03) << 13,
	dSPIN_PWM_DIV_5 = (0x04) << 13,
	dSPIN_PWM_DIV_6 = (0x05) << 13,
	dSPIN_PWM_DIV_7 = (0x06) << 13
} dSPIN_F_PWM_INT;

/**
 * PWM clock multiplier
 */
typedef enum {
	dSPIN_PWM_MUL_0_625 = (0x00) << 10,
	dSPIN_PWM_MUL_0_75 = (0x01) << 10,
	dSPIN_PWM_MUL_0_875 = (0x02) << 10,
	dSPIN_PWM_MUL_1 = (0x03) << 10,
	dSPIN_PWM_MUL_1_25 = (0x04) << 10,
	dSPIN_PWM_MUL_1_5 = (0x05) << 10,
	dSPIN_PWM_MUL_1_75 = (0x06) << 10,
	dSPIN_PWM_MUL_2 = (0x07) << 10
} dSPIN_F_PWM_DEC;

///@}

/** @name STATUS register */
///@{

#define dSPIN_STATUS_HIZ                     0x0001 ///< high when bridges are in HiZ mode
#define dSPIN_STATUS_BUSY                    0x0002 ///< mirrors BUSY pin
#define dSPIN_STATUS_SW_F                    0x0004 ///< low when switch open, high when closed
#define dSPIN_STATUS_SW_EVN                  0x0008 ///< active high, set on switch falling edge,
#define dSPIN_STATUS_DIR                     0x0010 ///< Indicates current motor direction.
#define dSPIN_STATUS_MOT_STATUS				0x0060 ///< Motor status
#define dSPIN_STATUS_NOTPERF_CMD             0x0080 ///< Last command not performed.
#define dSPIN_STATUS_WRONG_CMD               0x0100 ///< Last command not valid.
#define dSPIN_STATUS_UVLO                    0x0200 ///< Undervoltage lockout is active
#define dSPIN_STATUS_TH_WRN                  0x0400 ///< Thermal warning
#define dSPIN_STATUS_TH_SD                   0x0800 ///< Thermal shutdown
#define dSPIN_STATUS_OCD                     0x1000 ///< Overcurrent detected
#define dSPIN_STATUS_STEP_LOSS_A             0x2000 ///< Stall detected on A bridge
#define dSPIN_STATUS_STEP_LOSS_B             0x4000 ///< Stall detected on B bridge
#define dSPIN_STATUS_SCK_MOD                 0x8000 ///< Step clock mode is active

/**
 * Motor status values
 */
typedef enum {
	dSPIN_STATUS_MOT_STATUS_STOPPED = 0, 				///< Motor stopped
	dSPIN_STATUS_MOT_STATUS_ACCELERATION = 0x01 << 5, 	///< Motor accelerating
	dSPIN_STATUS_MOT_STATUS_DECELERATION = 0x02 << 5, 	///< Motor decelerating
	dSPIN_STATUS_MOT_STATUS_CONST_SPD = 0x03 << 5		///< Motor at constant speed
} dSPIN_MOT_STATUS;

///@}

/**
 * Motor directions
 */
typedef enum {
	dSPIN_DIR_REV = 0x00,
	dSPIN_DIR_FWD = 0x01
} dSPIN_DIRECTION;


/**
 * Action options
 */
typedef enum {
	dSPIN_ACTION_RESET = 0x00,
	dSPIN_ACTION_COPY = 0x01
} dSPIN_ACTION;

/**
 * Factory reset configuration register value
 */

#define dSPIN_FACT_RESET_CONFIG	0x2E88

#endif /* DSPIN_DEFS_H_ */
